(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/forms')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', '@angular/forms'], factory) :
	(factory((global['ngx-combo-datepicker'] = {}),global.ng.core,global.ng.common,global.ng.forms));
}(this, (function (exports,core,common,forms) { 'use strict';

var DateUtils = (function () {
    function DateUtils() {
    }
    /**
     * @param {?} myDate
     * @param {?} myTimezone
     * @return {?}
     */
    DateUtils.adjustTimezone = function (myDate, myTimezone) {
        var /** @type {?} */ offset = isNaN(myTimezone) ? new Date().getTimezoneOffset() : parseFloat(myTimezone) * 60;
        return new Date(myDate.getTime() + offset * 60 * 1000);
    };
    /**
     * @param {?} num
     * @return {?}
     */
    DateUtils.parseIntStrict = function (num) {
        return (num !== null && num !== '' && !isNaN(parseInt(num, 10))) ? parseInt(num, 10) : null;
    };
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    DateUtils.getMaxDate = function (month, year) {
        var /** @type {?} */ res = 31;
        if (month != null) {
            if (month === 4 || month === 6 || month === 9 || month === 11) {
                res = 30;
            }
            if (year !== null && month === 2) {
                res = ((year % 4 === 0) && (year % 100 !== 0)) || (year % 400 === 0) ? 29 : 28;
            }
        }
        return res;
    };
    /**
     * @param {?} myDate
     * @param {?} myTimezone
     * @return {?}
     */
    DateUtils.parseDate = function (myDate, myTimezone) {
        var /** @type {?} */ res = null;
        if (myDate !== undefined && myDate !== null) {
            if (myDate instanceof Date) {
                res = myDate;
            }
            else {
                if (typeof myDate === 'number' || typeof myDate === 'string') {
                    // Parse date.
                    if (typeof myDate === 'number') {
                        res = new Date(parseInt(myDate.toString(), 10));
                    }
                    else {
                        res = new Date(myDate);
                    }
                    // Adjust timezone.
                    res = DateUtils.adjustTimezone(res, myTimezone);
                }
            }
        }
        return res;
    };
    return DateUtils;
}());
var DEFAULT_VALUE_ACCESSOR = {
    provide: forms.NG_VALUE_ACCESSOR,
    useExisting: core.forwardRef(function () { return ComboDatepickerComponent; }),
    multi: true
};
var DEFAULT_VALIDATOR = {
    provide: forms.NG_VALIDATORS,
    useExisting: core.forwardRef(function () { return ComboDatepickerComponent; }),
    multi: true,
};
var ComboDatepickerComponent = (function () {
    function ComboDatepickerComponent() {
        this.attrsDate = {};
        this.attrsMonth = {};
        this.attrsYear = {};
        this.visible = [true, true, true];
        this.selects = {
            d: { visible: true },
            m: { visible: true },
            y: { visible: true }
        };
        this.monthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        this.propagateChange = function () { };
    }
    /**
     * @return {?}
     */
    ComboDatepickerComponent.prototype.ngOnInit = function () {
        // Initialize model.
        this.ngModel = DateUtils.parseDate(this.ngModel, this.timezone);
        // Initialize attributes variables.
        this.selects.d.attrs = this.attrsDate;
        this.selects.m.attrs = this.attrsMonth;
        this.selects.y.attrs = this.attrsYear;
        if (typeof this.disabled === 'boolean') {
            this.selects.d.disabled = this.disabled;
            this.selects.m.disabled = this.disabled;
            this.selects.y.disabled = this.disabled;
        }
        else if (this.disabled instanceof Array && this.disabled.length === 3) {
            this.selects.d.disabled = this.disabled[0];
            this.selects.m.disabled = this.disabled[1];
            this.selects.y.disabled = this.disabled[2];
        }
        if (typeof this.visible === 'boolean') {
            this.selects.d.visible = this.visible;
            this.selects.m.visible = this.visible;
            this.selects.y.visible = this.visible;
        }
        else if (this.visible instanceof Array && this.visible.length === 3) {
            this.selects.d.visible = this.visible[0];
            this.selects.m.visible = this.visible[1];
            this.selects.y.visible = this.visible[2];
        }
        // Verify if initial date was defined.
        var /** @type {?} */ initDate = DateUtils.parseDate(this.date, this.timezone);
        if (initDate != null) {
            this.ngModel = initDate;
        }
        // Initialize order.
        if (typeof this.order !== 'string') {
            this.order = 'dmy'.split('');
        }
        else {
            this.order = this.order.toLowerCase().split('');
        }
        // Initialize minimal and maximum values.
        this._minDate = DateUtils.parseDate(this.minDate, this.timezone);
        if (this.minDate == null) {
            var /** @type {?} */ now = new Date();
            this._minDate = new Date(now.getFullYear() - 100, now.getMonth(), now.getDate(), now.getHours(), now.getMinutes(), now.getSeconds(), now.getMilliseconds());
        }
        this._maxDate = DateUtils.parseDate(this.maxDate, this.timezone);
        if (this._maxDate == null) {
            this._maxDate = new Date();
        }
        // Verify if selected date is in the valid range.
        if (this.ngModel && this.ngModel < this._minDate) {
            this.ngModel = this._minDate;
        }
        if (this.ngModel && this.ngModel > this._maxDate) {
            this.ngModel = this._maxDate;
        }
        if (this.months !== undefined && this.months !== null) {
            if (typeof this.months === 'string') {
                var /** @type {?} */ months = this.months.split(',');
                if (months.length === 12) {
                    this.monthNames = months;
                }
            }
            if (Array.isArray(this.months) && this.months.length === 12) {
                this.monthNames = this.months;
            }
        }
        this.updatePlaceholders();
        this.updateMonthList('');
        this.updateYearList('');
        this.updateDateList('', '');
        this.writeValue(this.ngModel);
    };
    /**
     * @return {?}
     */
    ComboDatepickerComponent.prototype.onChange = function () {
        var /** @type {?} */ res = null;
        // Check that the three combo boxes have values.
        if (this.selects.d.value && !isNaN(this.selects.m.value) &&
            this.selects.y.value != null && this.selects.y.value) {
            var /** @type {?} */ maxDay = DateUtils.getMaxDate(this.selects.m.value + 1, this.selects.m.value);
            var /** @type {?} */ hours = 0;
            var /** @type {?} */ minutes = 0;
            var /** @type {?} */ seconds = 0;
            var /** @type {?} */ milliseconds = 0;
            if (this.ngModel != null) {
                hours = this.ngModel.getHours();
                minutes = this.ngModel.getMinutes();
                seconds = this.ngModel.getSeconds();
                milliseconds = this.ngModel.getMilliseconds();
            }
            // Remove already selected day & date if day is  greater than maxday then form invalid value also resets
            if (this.selects.d.value > maxDay) {
                this.selects.d.value = null;
            }
            else {
                res = new Date(this.selects.y.value, this.selects.m.value, this.selects.d.value > maxDay ? maxDay : this.selects.d.value, hours, minutes, seconds, milliseconds);
            }
        }
        // Disable placeholders after selecting a value.
        if (this.placeHolders) {
            if (this.selects.y.value) {
                this.placeHolders[0].disabled = true;
            }
            if (this.selects.m.value) {
                this.placeHolders[1].disabled = true;
            }
            if (this.selects.d.value) {
                this.placeHolders[2].disabled = true;
            }
        }
        // Hide or show days and months according to the min and max dates.
        this.updateMonthList(this.selects.y.value);
        this.updateYearList(this.selects.m.value);
        this.updateDateList(this.selects.m.value, this.selects.y.value);
        this.propagateChange(res);
    };
    /**
     * @param {?} value
     * @return {?}
     */
    ComboDatepickerComponent.prototype.writeValue = function (value) {
        if (value) {
            this.selects.d.value = value.getDate();
            this.selects.m.value = value.getMonth();
            this.selects.y.value = value.getFullYear();
        }
        else {
            if (typeof this.visible === 'boolean') {
                this.selects.d.value = this.visible ? null : 1;
                this.selects.m.value = this.visible ? null : 0;
                this.selects.y.value = this.visible ? null : new Date().getFullYear();
            }
            else if (this.visible instanceof Array && this.visible.length === 3) {
                this.selects.d.value = this.visible[0] ? null : 1;
                this.selects.m.value = this.visible[1] ? null : 0;
                this.selects.y.value = this.visible[2] ? null : new Date().getFullYear();
            }
            if (this.placeHolders) {
                this.placeHolders[0].disabled = false;
                this.placeHolders[1].disabled = false;
                this.placeHolders[2].disabled = false;
            }
        }
        this.updateMonthList(this.selects.y.value);
        this.updateYearList(this.selects.m.value);
        this.updateDateList(this.selects.m.value, this.selects.y.value);
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    ComboDatepickerComponent.prototype.registerOnChange = function (fn) {
        this.propagateChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    ComboDatepickerComponent.prototype.registerOnTouched = function (fn) {
    };
    /**
     * @param {?} c
     * @return {?}
     */
    ComboDatepickerComponent.prototype.validate = function (c) {
        return this.selects.d.value && this.selects.d.value && this.selects.y.value ?
            null : { required: true };
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    ComboDatepickerComponent.prototype.ngOnChanges = function (changes) {
        if (changes.placeholder && !changes.placeholder.isFirstChange()) {
            this.updatePlaceholders();
            this.updateMonthList(this.selects.y.value);
            this.updateYearList(this.selects.m.value);
            this.updateDateList(this.selects.m.value, this.selects.y.value);
        }
    };
    /**
     * @param {?} year
     * @return {?}
     */
    ComboDatepickerComponent.prototype.updateMonthList = function (year) {
        // Parse parameter.
        year = DateUtils.parseIntStrict(year);
        // Some months can not be chosen if the year matches with the year of the minimum or maximum dates.
        var /** @type {?} */ start = year !== null && year === this._minDate.getFullYear() ? this._minDate.getMonth() : 0;
        var /** @type {?} */ end = year !== null && year === this._maxDate.getFullYear() ? this._maxDate.getMonth() : 11;
        // Generate list.
        this.selects.m.options = [];
        if (this.placeHolders) {
            this.selects.m.options.push(this.placeHolders[1]);
        }
        for (var /** @type {?} */ i = start; i <= end; i++) {
            this.selects.m.options.push({ value: i, name: this.monthNames[i] });
        }
    };
    /**
     * @param {?=} month
     * @return {?}
     */
    ComboDatepickerComponent.prototype.updateYearList = function (month) {
        // Parse parameter.
        month = DateUtils.parseIntStrict(month);
        this.selects.y.options = [];
        var /** @type {?} */ isReverse = typeof this.yearOrder === 'string' && this.yearOrder.indexOf('des') === 0;
        for (var /** @type {?} */ i = this._minDate.getFullYear(); i <= this._maxDate.getFullYear(); i++) {
            var /** @type {?} */ now = new Date();
            if (month + 1 < this._minDate.getMonth() && now.getFullYear() === i && month !== null &&
                (isReverse && month > now.getMonth() || !isReverse && month < now.getMonth())) {
                continue;
            }
            this.selects.y.options.push({ value: i, name: i });
        }
        // Verify if the order of the years must be reversed.
        if (isReverse) {
            this.selects.y.options.reverse();
        }
        // Prepend the years placeholder
        if (this.placeHolders) {
            this.selects.y.options.unshift(this.placeHolders[0]);
        }
    };
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    ComboDatepickerComponent.prototype.updateDateList = function (month, year) {
        // Parse parameters.
        month = DateUtils.parseIntStrict(month);
        year = DateUtils.parseIntStrict(year);
        // Start date is 1, unless the selected month and year matches the minimum date.
        var /** @type {?} */ start = 1;
        if (month !== null && month === this._minDate.getMonth() &&
            year !== null && year === this._minDate.getFullYear()) {
            start = this._minDate.getDate();
        }
        // End date is 30 or 31 (28 or 29 in February), unless the selected month and year matches the maximum date.
        var /** @type {?} */ end = DateUtils.getMaxDate(month !== null ? (month + 1) : null, year);
        if (month !== null && month === this._maxDate.getMonth() &&
            year !== null && year === this._maxDate.getFullYear()) {
            end = this._maxDate.getDate();
        }
        // Generate list.
        this.selects.d.options = [];
        if (this.placeHolders) {
            this.selects.d.options.push(this.placeHolders[2]);
        }
        for (var /** @type {?} */ i = start; i <= end; i++) {
            this.selects.d.options.push({
                value: i,
                name: i
            });
        }
    };
    /**
     * @return {?}
     */
    ComboDatepickerComponent.prototype.updatePlaceholders = function () {
        this.placeHolders = null;
        if (this.placeholder !== undefined && this.placeholder !== null &&
            (typeof this.placeholder === 'string' || Array.isArray(this.placeholder))) {
            var /** @type {?} */ holders = typeof this.placeholder === 'string' ?
                this.placeholder.split(',') : this.placeholder;
            if (holders.length === 3) {
                this.placeHolders = [];
                for (var _i = 0, holders_1 = holders; _i < holders_1.length; _i++) {
                    var h = holders_1[_i];
                    this.placeHolders.push({ value: null, name: h, disabled: false });
                }
            }
        }
    };
    return ComboDatepickerComponent;
}());
ComboDatepickerComponent.decorators = [
    { type: core.Component, args: [{
                selector: 'ngx-combo-datepicker',
                template: "\n    <ng-container *ngFor=\"let type of order\">\n      <select *ngIf=\"selects[type].visible\"\n              [(ngModel)]=\"selects[type].value\"\n              (change)=\"onChange($event)\"\n              [disabled]=\"selects[type].disabled\"\n              customAttrs [attrs]=\"selects[type].attrs\">\n        <option *ngFor=\"let option of selects[type].options\"\n                [ngValue]=\"option.value\"\n                [disabled]=\"option.disabled\">\n          {{ option.name }}\n        </option>\n      </select>\n    </ng-container>\n  ",
                providers: [
                    DEFAULT_VALUE_ACCESSOR,
                    DEFAULT_VALIDATOR
                ]
            },] },
];
/**
 * @nocollapse
 */
ComboDatepickerComponent.ctorParameters = function () { return []; };
ComboDatepickerComponent.propDecorators = {
    'ngModel': [{ type: core.Input },],
    'date': [{ type: core.Input },],
    'minDate': [{ type: core.Input },],
    'maxDate': [{ type: core.Input },],
    'months': [{ type: core.Input },],
    'order': [{ type: core.Input },],
    'attrsDate': [{ type: core.Input },],
    'attrsMonth': [{ type: core.Input },],
    'attrsYear': [{ type: core.Input },],
    'yearOrder': [{ type: core.Input },],
    'timezone': [{ type: core.Input },],
    'placeholder': [{ type: core.Input },],
    'ngRequired': [{ type: core.Input },],
    'disabled': [{ type: core.Input },],
    'visible': [{ type: core.Input },],
};
var AttributesDirective = (function () {
    /**
     * @param {?} element
     * @param {?} renderer
     */
    function AttributesDirective(element, renderer) {
        this.element = element;
        this.renderer = renderer;
    }
    /**
     * @return {?}
     */
    AttributesDirective.prototype.ngOnInit = function () {
        var _this = this;
        Object.keys(this.attrs).forEach(function (key) { return _this.renderer.setAttribute(_this.element.nativeElement, key, _this.attrs[key]); });
    };
    return AttributesDirective;
}());
AttributesDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[customAttrs]'
            },] },
];
/**
 * @nocollapse
 */
AttributesDirective.ctorParameters = function () { return [
    { type: core.ElementRef, },
    { type: core.Renderer2, },
]; };
AttributesDirective.propDecorators = {
    'attrs': [{ type: core.Input },],
};
var ComboDatepickerModule = (function () {
    function ComboDatepickerModule() {
    }
    return ComboDatepickerModule;
}());
ComboDatepickerModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [
                    common.CommonModule,
                    forms.FormsModule
                ],
                declarations: [
                    ComboDatepickerComponent,
                    AttributesDirective,
                ],
                exports: [
                    ComboDatepickerComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
ComboDatepickerModule.ctorParameters = function () { return []; };

exports.ComboDatepickerModule = ComboDatepickerModule;
exports.ɵc = ComboDatepickerComponent;
exports.ɵb = DEFAULT_VALIDATOR;
exports.ɵa = DEFAULT_VALUE_ACCESSOR;
exports.ɵd = AttributesDirective;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-combo-datepicker.umd.js.map
